<?php
interface IAboCaptcha
{
    //public function render($param = false);
    public function isValidCode($code, $param = false);
    public function clean($param = false);
}

class AboCaptchaFactory {
    public function get($name, $conf = false)
    {
        $ret = false;
        switch ($name) {
            case 'simple':
                $ret = new AboCaptchaSimple($conf);
                break;
            default:
                throw new Exception('Not defined for the class factory');
                break;
        }
        return $ret;
    }
}

abstract class AboCaptcha implements IAboCaptcha {
    
    private static $sDefLength = 5;
    private static $sDefChars = '2345678wrtysdfhjzvn';
    private static $sDefSessionKey = 'captcha_key';
    
    abstract public function render($param = false);
    
    public function isValidCode($code, $param = false)
    {
        $k = $param && isset($param['key']) ? $param['key'] : false;
        $key = self::getSessionKey($k);
        $ret = isset($_SESSION[$key]) && $code == $_SESSION[$key] ? true : false;
        if (! $ret) {
            self::clean($key);
        }
        return $ret;
    }
    
    public function clean($param = false)
    {
        $k = $param && isset($param['key']) ? $param['key'] : false;
        unset($_SESSION[self::getSessionKey($k)]);
    }
    
    public static function setCodeLength($length)
    {
        self::$sDefLength = $length;
    }
    
    public function setSessionKey($key)
    {
        if ($key) {
            self::$sDefSessionKey = $key;
        }
        
        return self::$sDefSessionKey;
    }
    
    protected function getSessionKey($key = false)
    {
        return $key ? 'cpch'.md5($key) : self::$sDefSessionKey;
    } 
    
    protected function generateCode($length = 0, $chars = '')
    {
        $length = $length > 1 ? $length : self::$sDefLength;
        $chars = $chars ? $chars : self::$sDefChars;
        $ret = array();
        for($i = 0; $i < $length; $i++)
        {
            $ret[] = substr($chars, rand(0, strlen($chars) - 1), 1);
        }
        
        return implode('', $ret);
    }
    
    protected function saveCode($code, $param = false)
    {
        $_SESSION[self::getSessionKey($param['key'])] = $code;
    }
}

class AboCaptchaSimple extends AboCaptcha {
    private $conf = array(
        'image_type' => 'png',
        'image_width' => 110,
        'image_height' => 32,
        'font_size' => 18,
        'font_depth' => 2,
        'grid_size' => 7,
        'bgcolor' => array(98, 118, 144),
        'color' => array(200, 210, 40 ),
        'color2' => array(160, 60, 150),
        'font_uri' => array(
            'inc/captcha/arialbd.ttf',
            'inc/captcha/comicbd.ttf',
            'inc/captcha/courbd.ttf',
            'inc/captcha/timesbd.ttf'
        )
    );
    private $img_render = false;
    
    public function __construct($conf = false)
    {
        if (is_array($conf) && !empty($conf)) {
            foreach ($conf as $key => $val) {
                if ($val) {
                    $this->conf[$key] = $val;
                }
            }
        }
        if ($this->conf['random_bgcolor']) {
            $this->conf['bgcolor'] = array(rand(0, 255), rand(0, 255), rand(0, 255));
        } else if (! is_array($this->conf['bgcolor'])) {
            $this->conf['bgcolor'] = array(
                hexdec(substr($this->conf['bgcolor'], 0, 2)),
                hexdec(substr($this->conf['bgcolor'], 2, 2)),
                hexdec(substr($this->conf['bgcolor'], 4, 2))
            );
        }
        if ($this->conf['random_color']) {
            $this->conf['color'] = array(rand(0, 255), rand(0, 255), rand(0, 255));
        } else if (! is_array($this->conf['color'])) {
            $this->conf['color'] = array(
                    hexdec(substr($this->conf['color'], 0, 2)),
                    hexdec(substr($this->conf['color'], 2, 2)),
                    hexdec(substr($this->conf['color'], 4, 2))
            );
        }
        if ($this->conf['random_color']) {
            $this->conf['color2'] = array(rand(0, 255), rand(0, 255), rand(0, 255));
        } else if (! is_array($this->conf['color2'])) {
            $this->conf['color2'] = array(
                    hexdec(substr($this->conf['color2'], 0, 2)),
                    hexdec(substr($this->conf['color2'], 2, 2)),
                    hexdec(substr($this->conf['color2'], 4, 2))
            );
        }
        require_once(RP.'inc/captcha/lib.image.php');
        
        if ('gif' == $this->conf['image_type']) {
            $this->img_render = new CryptGif($this->conf['image_width'], $this->conf['image_height']);
        } else if ('jpg' == $this->conf['image_type']) {
            $this->img_render = new CryptJpeg($this->conf['image_width'], $this->conf['image_height']);
        } else {
            $this->img_render = new CryptPng($this->conf['image_width'], $this->conf['image_height']);
        }
    }
    
    public function render($param = false) {
        $code = self::generateCode();
        self::saveCode($code, $param);
        if($this->img_render->create($this->conf['bgcolor'])) {
            if ($this->conf['gradient_effect']) {
                $this->img_render->apply(new GradientEffect($this->conf['bgcolor'][0], $this->conf['bgcolor'][1], $this->conf['bgcolor'][2]));
            }
            if ($this->conf['grid_effect']) {
                $this->img_render->apply(new GridEffect($this->conf['grid_size'], $this->conf['color'][0], $this->conf['color'][1], $this->conf['color'][2]));
            }
            if ($this->conf['dot_effect']) {
                $this->img_render->apply(new DotEffect($this->conf['color']));
            }
            $t = new TextEffect($code, $this->conf['font_size'], $this->conf['font_depth'], $this->conf['color2'], $this->conf['color']);
            foreach($this->conf['font_uri'] as $k => $v) {
                $t->addFont(RP.$v);
            }
            $this->img_render->apply($t);
            $this->img_render->render();
        }
    }
}
?>